#ifndef	lint
static char sccsid[] = "$Id: SIGCHLD_handler.c,v 1.1 1995/01/24 05:16:42 sydd Exp $ SyddWare";
#endif
/*****************************************************************************
* Classification:    Unclassified
*
* Project:           SDRTOOL
*
* Module Name:       $Source: /home/sydd/sdrtool/RCS/SIGCHLD_handler.c,v $
*
* Description:
*   This function handles SIGCHLD events caused by child processes terminating,
*   both abnormally and normally.  It marks the process as terminated, checks
*   return status for normal termination, recorded state, and retry count, and
*   flags for restart worthy processes in their corresponding table entry.
*   It is installed via the signal() command to field child termination events.
*
* Calling Sequence:
*   SIGCHLD_handler (int signal_number)
*
*   Inputs: Parms are defined by the UNIX signal(3) manpage
*       signal_number - see signal(3) manpage - should be SIGCHLD
*
*   Outputs:
*       None - modifies global process tables
*
*   Returns:
*       None
*
* Error Conditions and Actions:
*   N/A
*
* Side Effects:
*   N/A
*
* Limitations:
*	N/A
*
* Notes:
*   Guts are from REUSE-CODE.
*
* Revision History
*   DATE    S/W ENGINEER     DESCRIPTION OF WORK
* --------  ---------------  ------------------------------------------------
* 09/13/93  J.R.(Sydd)Souza  Created
******************************************************************************/
/*
**	Includes
*/ 
#include <sys/time.h>
#ifndef RUSAGE_SELF
#	include <sys/resource.h>
#endif  /* RUSAGE_SELF */
#include <sys/wait.h>
#include <signal.h>

#include "sdr_.h"	/* for this routine's prototype */
#include "sdr_g.h"	/* for Process_table */

#ifdef	COMMENT_OUT
/*
**	non-ANSI Prototypes
*/
pid_t wait3
    PROTO(( int *wait_status, int options, int *rusage ));
#endif	/* COMMENT_OUT */

/*
**	Function SIGCHLD_handler
*/
/*
**	This is the notify process when execlp commands complete to avoid Zombies
*/
void	SIGCHLD_handler ( signal_number )
	int signal_number ;
{
	/*
	**	Local variables
	*/
    char        *text;
    XmString            origstring;
    XmStringContext     context;
    XmStringCharSet     charset;
    XmStringDirection   direction;
    Boolean             separator;
	int	wait_status;
#ifdef	COMMENT_OUT
	int	rusage;
#else	/* COMMENT_OUT */
	struct rusage *rusage;
#endif	/* COMMENT_OUT */
	pid_t   		terminated_process;
	int				icnt;
	int				options = WNOHANG;
	int				signal_mask;
	int				notperform = -1;
	int				notvalidate = -1;
    char        	system_command[256];

	/*
	**	Function SIGCHLD_handler body
	*/
	/*
    **  Prevent SIGCHLD events while in this function
    */
    BLOCK_SIGNAL(SIGCHLD,signal_mask);
	DIAG3(SIGCHLD_handler:\
,signal_number,d,-,code,d,-,sigcontextptr,lx);
	/* check if a child died for some reason */
	terminated_process = wait3 (&wait_status, options, (struct rusage *)0);
	if (terminated_process != -1)
	{
		for (icnt=0;Process_table[icnt].extension &&
		   	Process_table[icnt].pid != terminated_process;
			++icnt);
		if (Process_table[icnt].extension)
		{
			Process_table[icnt].pid = (pid_t)-1;
			DIAG1(SIGCHLD_handler: child terminated\
,terminated_process,d);
            if (strcmp(Process_table[icnt].extension,HELP_FILE_EXT) &&
				Process_table[icnt].id[0] == ORIGINAL_PREFIX)
            {
                color_buttons((char *)Process_table[icnt].id);
				if (!(notvalidate = strcmp(Process_table[icnt].extension,
						VALIDATED_EXT)) ||
					!(notperform = strcmp(Process_table[icnt].extension,
						PERFORMED_EXT)))
				{
            		/* change the label back so they know it's done */
            		XtVaGetValues(!notvalidate ?
						Widget_list[k_dovalidatepushButton] :
						Widget_list[k_doperformpushButton],
                		XmNlabelString, &origstring, NULL);
            		XmStringInitContext(&context,origstring);
            		XmStringGetNextSegment(context,
                		&text, &charset, &direction, &separator);
            		XmStringFreeContext(context);
            		XmStringFree(origstring);
					/* it has ' [ ' before it & ' ] ' after */
					if (text[1] == '[')
					{
            			strcpy(system_command, text+3);
						if ((text = strstr(system_command," ] ")))
							*text = '\0';
            			origstring = XmStringCreateLtoR(system_command,
                			XmFONTLIST_DEFAULT_TAG);
            			XtVaSetValues(!notvalidate ?
                        	Widget_list[k_dovalidatepushButton] :
                        	Widget_list[k_doperformpushButton],
                			XmNlabelString, origstring, NULL);
            			XmStringFree(origstring);
					}
				}
            }
		}
	}
	INSTALL_SIGNAL_HANDLER(SIGCHLD, SIGCHLD_handler);
	UNBLOCK_SIGNAL(signal_mask);
	return;
} /* end function : SIGCHLD_handler */
/* end file : $Source: /home/sydd/sdrtool/RCS/SIGCHLD_handler.c,v $ */

