#ifndef	lint
static char rcsid[] = "$Id: apply_abort_callback.c,v 1.2 1995/01/29 21:18:38 sydd Exp $ SyddWare";
#endif
/*****************************************************************************
* Classification:    Unclassified
*
* Project:           SDRTOOL
*
* Module Name:       $Source: /home/sydd/sdrtool/RCS/apply_abort_callback.c,v $
*
* Description:
*   This is a PushButton Callback called when the user presses the Apply or
*	Abort button at the bottom of the screen.
*		Abort performs the following functions:
*		- Clear all fields to "initial" state
*		- Delete any <pid><extension> files found in /tmp
*		- If ID field was not NEW, delete any <number><extension> files
*		found in /tmp. [Yes this does not guarantee full cleanup.]
*		Apply performs the following functions:
*		- If any files (except the help file) are still open, beep,
*		display a message & return
*		- If the Title field is empty, beep, display a message & return
*		- If NEW, get the next 'N' number, rename all files found in
*		/tmp named <pid><extension> to <newnumber><extension>, check in
*		the files, insert the new records in the database, mails a message
*		to originator & CM & refreshes screen with contents of a query
*		from the database (to show what actually made it in).
*		- If not NEW, check the userid: if CM, originator or person to
*		which it is assigned, files are updated.  For all users, Comments
*		file is updated.
*
* Calling Sequence:
*	void apply_abort_callback(
*		Widget w,
*		int *which_one,
*		XmPushButtonCallbackStruct *call);
*
*   Inputs:
*       w - id of widget pressed
*		which_one - pointer to tag value
*		call - pointer to event structure on event
*
*   Outputs:
*       None
*
*   Returns:
*       None
*
* Error Conditions and Actions:
*   N/A
*
* Side Effects:
*   N/A
*
* Limitations:
*   If we used a new number and then the database insert fails, we do not
*	currently back off that number - leaving it unused.  This is a bug.
*
* Notes:
*   N/A
*
* Revision History
*   DATE    S/W ENGINEER     DESCRIPTION OF WORK
* --------  ---------------  ------------------------------------------------
* 04/01/94  J.R.(Sydd)Souza  Created
******************************************************************************/
#include <stdlib.h>		/* for system() prototype */
#include "sdr_.h"		/* for this prototype */
#include "sdr_g.h"		/* for globals */

void
apply_abort_callback(w, which_one, call)
	Widget w;
	int *which_one;
	XmPushButtonCallbackStruct *call;
{
	char 		*current_value = (char *)0;
	char 		*lockfilename = (char *) 0;
	char 		*last_id;
	char 		*message_ptr = (char *)0;
	char		*this_id = (char *)0;
	char		*this_title = (char *)0;
	SDR_T		sdr;
	XtPointer	idvalid;
	STATUS_T	previous_state;
	ID_CODE_T 	tmp_id;
	BOOLEAN		insert_record = FALSE;
	BOOLEAN		no_file_updates = FALSE;
	char 		message_string[1024];
	char 		replace_script[256];
	int			icnt;

	replace_script[0] = '\0';
	switch (*which_one)
	{
		case k_applypushButton:
			/* check that no files are still being edited (Help file is ok) */
			for (icnt=0;Process_table[icnt].extension;++icnt)
				if (Process_table[icnt].pid != -1 &&
					strcmp(Process_table[icnt].extension,HELP_FILE_EXT))
				{
					BEEP_MESSAGE("Please finish editing files & exit editors");
					return;
				}

			/* check that the id field has been properly queried */
			XtVaGetValues(Widget_list[k_idtextField],
				XmNuserData, &idvalid,
				NULL);
			if (!idvalid)
			{
				current_value = XmTextFieldGetString(
					Widget_list[k_idtextField]);
				strcpy(tmp_id,current_value);
				strip_whitespace(tmp_id);
				if (!emptyline(tmp_id) &&
					strcmp(tmp_id,"NEW") &&
					strlen(tmp_id) == ID_CODE_SIZE)
				{
					cursor_mgmt(XtParent(w), TRUE);
					strcpy(sdr.id[0],tmp_id);
					if (get_current_record(&sdr) ||
						emptyline(sdr.id[0]))	/* i.e. not found */
					{
						BEEP_MESSAGE("SDR not found");
					}
					else
					{
						/* field is good */
						XtVaSetValues(Widget_list[k_idtextField],
							XmNuserData, True, NULL);
						FILL_IT_IN(sdr);
						if (!strcmp(username, SdrTool.cmaccountname))
						{
							if (!XtIsSensitive(Widget_list[k_priorityscale]))
								XtSetSensitive(Widget_list[k_priorityscale],
									True);
						}
						else if (XtIsSensitive(Widget_list[k_priorityscale]))
							XtSetSensitive(Widget_list[k_priorityscale],
								False);
					}
					cursor_mgmt(XtParent(w), FALSE);
					return;
				}
				else if (strcmp(tmp_id,"NEW"))
				{
					BEEP_MESSAGE("Fill in SDR first");
					return;
				}
			}
			/* check that title has been entered */
			this_id = XmTextFieldGetString(Widget_list[ k_idtextField ]);
			this_title = XmTextFieldGetString(Widget_list[ k_titletextField ]);

			if (!emptyline(this_id) &&
				!emptyline(this_title))
			{
				cursor_mgmt(XtParent(w), TRUE);	/* cursor to a clock */
				no_file_updates = FALSE;
				if (!extract_from_window(&sdr,&insert_record,&previous_state,							&lockfilename))
				{
					/* for new ones files are named with pid in Tempdir */
					tmp_id[0] = '\0';
					if (insert_record)
					{
						sprintf(tmp_id,"%d",getpid());
						/*
						**	Must now rename new files from pid to new id
						**	NOTE: This only renames them in place.
						*/
                        CHECK_AND_REPLACE(Tempdir,sdr.id[0],Tempdir,tmp_id,
                            BUILD_EXT, TRUE);
                        CHECK_AND_REPLACE(Tempdir,sdr.id[0],Tempdir,tmp_id,
                            DESCRIPTION_EXT, TRUE);
                        CHECK_AND_REPLACE(Tempdir,sdr.id[0],Tempdir,tmp_id,
                            FILES_EXT, TRUE);
                        CHECK_AND_REPLACE(Tempdir,sdr.id[0],Tempdir,tmp_id,
                            COMMENTS_EXT, TRUE);
                        CHECK_AND_REPLACE(Tempdir,sdr.id[0],Tempdir,tmp_id,
                            PROBLEM_EXT, TRUE);
                        CHECK_AND_REPLACE(Tempdir,sdr.id[0],Tempdir,tmp_id,
                            RECREATE_EXT, TRUE);
                        CHECK_AND_REPLACE(Tempdir,sdr.id[0],Tempdir,tmp_id,
                            RESOLUTION_EXT, TRUE);
                        CHECK_AND_REPLACE(Tempdir,sdr.id[0],Tempdir,tmp_id,
                            SOLUTION_EXT, TRUE);
                        CHECK_AND_REPLACE(Tempdir,sdr.id[0],Tempdir,tmp_id,
                            TESTING_EXT, TRUE);
						strcpy(tmp_id,sdr.id[0]);
					}
					else if (!strcmp(username,sdr.assigned_to) ||
						!strcmp(username,sdr.originator) ||
						!strcmp(username,SdrTool.cmaccountname))
						strcpy(tmp_id,sdr.id[0]);
					else
						no_file_updates = TRUE;

					/* Everyone is allowed to change the comments file */
					sprintf(replace_script,"%s/.replace_files",SdrTool.sdrpath);
					if (access(replace_script,NOTEXECUTABLE))
					{
						BEEP_MESSAGE(
							"Files not replaced - .replace_files missing");
						replace_script[0] = '\0'; /* indicate it to posterity */
					}
					else
					{
						sprintf(message_string,
							"/bin/sh %s %s %s %s",
							replace_script, sdr.id[0], Tempdir, COMMENTS_EXT);

						/*
						**	Note: I KNOW system() is less than perfect but we
						**	need to wait for completion
						*/
						system(message_string);
					}

					/* Reset value of k_idtextField to sdr.id[0] */
					XmTextFieldSetString( Widget_list[ k_idtextField ],
							 sdr.id[0]);

					/* based upon transition, change date fields  */
					/* CM needs to fill in date due on assignment */
					if (!strcmp(username, SdrTool.cmaccountname) &&
						previous_state != sdr.status)
					{
						switch (sdr.status)
						{
							case ASSESS:
							case ASSIGNED:
								/* change date to today */
								CURRENT_DATE(sdr.assigned);
								XmTextFieldSetString(
									Widget_list[ k_assigndatetextField ],
									sdr.assigned);
								if (emptyline(sdr.due))
								{
									/* can't stop now - use today & beep */
									CURRENT_DATE(sdr.due);
									XmTextFieldSetString(
										Widget_list[ k_duedatetextField ],
										sdr.due);
									BEEP_MESSAGE(
										"Due Date is set to todays date");
								}
								break;

							case REJECTED:
							case RESOLVED:
							case PROMOTE:
								/* change date to today */
								CURRENT_DATE(sdr.closed);
								XmTextFieldSetString(
									Widget_list[ k_closedatetextField ],
									sdr.closed);
								break;

							default:
								DO_NOTHING;
								break;
						}
					}

					/* call database proc to insert/replace database record*/
					if (update_database(insert_record, &sdr))
					{
						/* update failed? then what's in there? */
						BEEP_MESSAGE("Database update failed");
						if (get_current_record(&sdr) ||
            				emptyline(sdr.id[0]))   /* not found */
        				{
							unlock_idfile(lockfilename);
            				BEEP_MESSAGE(
					"Database update failed; SDR by that ID does not exist");
							cursor_mgmt(XtParent(w), FALSE);
            				return;
        				}
						else
						{
						    /*
							**	refresh screen with things in db so they see
							**	what didn't get updated.
							*/
    						FILL_IT_IN(sdr);
						}
					}
					else
					{
						/* get id again - this time make it stick */
						get_next_id(ORIGINAL_PREFIX,tmp_id,TRUE);
					}
					unlock_idfile(lockfilename);

					/* set id text field "good" marker */
					XtVaSetValues(Widget_list[k_idtextField],
						XmNuserData, True, NULL);

					/* adjust sensitivity of scale */
                    if (!strcmp(username,SdrTool.cmaccountname))
                    {
                        if (!XtIsSensitive(Widget_list[k_priorityscale]))
                            XtSetSensitive(Widget_list[k_priorityscale],
                                True);
                    }
                    else if (XtIsSensitive(Widget_list[k_priorityscale]))
                        XtSetSensitive(Widget_list[k_priorityscale],
                            False);

					/* remove or move files from /tmp to real home */
					if (tmp_id[0])
					{
						/* already tried it once & failed, 1st char is null */
						if (!replace_script[0])
						{
							BEEP_MESSAGE(
							"Files not replaced - .replace_files missing");
						}
						else
						{
							/*
							**	replace any changed files using .replace_files
							**	script - this gives us the flexibilty to use
							**	SCCS to log changes to the files.
							**	We already did the COMMENTS_EXT file above.
							*/
							sprintf(message_string,
"/bin/sh %s %s %s %s %s %s %s %s %s %s %s",
	replace_script, tmp_id, Tempdir, BUILD_EXT, DESCRIPTION_EXT, FILES_EXT,
	PROBLEM_EXT, RECREATE_EXT, RESOLUTION_EXT, SOLUTION_EXT, TESTING_EXT);

							/*
							**	Note: We need to wait for completion before
							**	mailing copies to appropriate recipients because
							**	notify_user() appends the text from the real
							**	home of the files.
							*/
							system(message_string);
							color_buttons(tmp_id);
						}
					}
					else
					{
						/* let em know nothing was saved */
						BEEP_MESSAGE("Changed files, if any, were not saved - check your mail");

						strcpy(tmp_id,sdr.id[0]);
						/*
						**	package up any changed files using .savemail_files
						**	script which places any Tempdir files found for
						**	a given sdr that differ from the ones in their
						**	real homes into a tempfile & mails it to the user
						**	thereby saving them since that user is not allowed
						**	to change them.
						**	We already did the COMMENTS_EXT file above.
						*/
						sprintf(message_string,
"/bin/sh /devel/sdr/.savemail_files %s %s %s %s %s %s %s %s %s %s",
	tmp_id, Tempdir, BUILD_EXT, DESCRIPTION_EXT, FILES_EXT,
	PROBLEM_EXT, RECREATE_EXT, RESOLUTION_EXT, SOLUTION_EXT, TESTING_EXT);

						/*
						**	Note: We need to wait for completion before
						**	deleting them.
						*/
						system(message_string);
						color_buttons(tmp_id);

						/* lose any temp files created during this one */
						/* They could be named after this process if new... */
						sprintf(tmp_id,"%d",getpid());
						CHECK_AND_DELETE(Tempdir,tmp_id,DESCRIPTION_EXT);
						CHECK_AND_DELETE(Tempdir,tmp_id,BUILD_EXT);
						CHECK_AND_DELETE(Tempdir,tmp_id,FILES_EXT);
						CHECK_AND_DELETE(Tempdir,tmp_id,COMMENTS_EXT);
						CHECK_AND_DELETE(Tempdir,tmp_id,PROBLEM_EXT);
						CHECK_AND_DELETE(Tempdir,tmp_id,RECREATE_EXT);
						CHECK_AND_DELETE(Tempdir,tmp_id,RESOLUTION_EXT);
						CHECK_AND_DELETE(Tempdir,tmp_id,SOLUTION_EXT);
						CHECK_AND_DELETE(Tempdir,tmp_id,TESTING_EXT);
						CHECK_AND_DELETE(Tempdir,tmp_id,VALIDATED_EXT);
						CHECK_AND_DELETE(Tempdir,tmp_id,PERFORMED_EXT);
						/* ... or they could have a number assignment. */
						strcpy(tmp_id,this_id);
						CHECK_AND_DELETE(Tempdir,tmp_id,DESCRIPTION_EXT);
						CHECK_AND_DELETE(Tempdir,tmp_id,BUILD_EXT);
						CHECK_AND_DELETE(Tempdir,tmp_id,FILES_EXT);
						CHECK_AND_DELETE(Tempdir,tmp_id,COMMENTS_EXT);
						CHECK_AND_DELETE(Tempdir,tmp_id,PROBLEM_EXT);
						CHECK_AND_DELETE(Tempdir,tmp_id,RECREATE_EXT);
						CHECK_AND_DELETE(Tempdir,tmp_id,RESOLUTION_EXT);
						CHECK_AND_DELETE(Tempdir,tmp_id,SOLUTION_EXT);
						CHECK_AND_DELETE(Tempdir,tmp_id,TESTING_EXT);
						CHECK_AND_DELETE(Tempdir,tmp_id,VALIDATED_EXT);
						CHECK_AND_DELETE(Tempdir,tmp_id,PERFORMED_EXT);
						tmp_id[0] = '\0';	/* set it back just in case */
					}
	
					/* Mail messages appropriately */
					if (insert_record)
					{
						message_ptr = "\
NEW SDR: A New System Discrepancy Report was entered";
						notify_user(SdrTool.ccbmembers, &sdr, message_ptr);
						if (strcmp(username,SdrTool.cmaccountname))
							notify_user(SdrTool.cmaccountname, &sdr, message_ptr);
						if (!strstr(SdrTool.ccbmembers,username))
							notify_user(username, &sdr, message_ptr);
						RESET_MESSAGE("SDR Created");
					}
					else if (previous_state != sdr.status)
					{
						switch (sdr.status)
						{
							case NEW_CCB1:	/* back to CCB1 NOT NEW! */
								switch (previous_state)
								{
									case ASSESS:
										message_ptr = "\
CCB1: Assessment complete";
										break;

									case ASSIGNED:
										message_ptr = "\
CCB1: Potential reassignment";
										break;

									default:
										message_ptr = "CM CHANGED STATUS";
										break;
								}
								notify_user(SdrTool.ccbmembers, &sdr,
									message_ptr);
								if (strcmp(username,SdrTool.cmaccountname))
									notify_user(SdrTool.cmaccountname, &sdr,
										message_ptr);
								if (!strstr(SdrTool.ccbmembers,username))
									notify_user(username, &sdr, message_ptr);
								break;

							case ASSESS:
							case ASSIGNED:
								sprintf(message_string,"%s SDR # %s",
									(sdr.status == ASSESS) ? (char *)"\
SDR ASSESSMENT: Please assess" : (char *) "\
SDR ASSIGNED: You've been assigned", (char *)sdr.id[1]);
								notify_user(sdr.assigned_to, &sdr,
									message_string);
								break;

							case CCB2:
								sprintf(message_string,
									"CCB2: %s has completed changes",
									sdr.assigned_to);
								notify_user(SdrTool.cmaccountname, &sdr,
									message_string);
								notify_user(SdrTool.ccbmembers, &sdr,
									message_string);
								break;

							case RESOLVED:
							case REJECTED:
								message_ptr = (sdr.status == RESOLVED) ?
									(char *)"SDR RESOLVED" :
									(char *)"SDR HAS BEEN REJECTED";
								notify_user(SdrTool.ccbmembers, &sdr,
									message_ptr);
								if (!strstr(SdrTool.ccbmembers,
										sdr.assigned_to))
									notify_user(sdr.assigned_to, &sdr,
										message_ptr);
								if (strcmp(sdr.assigned_to,SdrTool.cmaccountname))
									notify_user(SdrTool.cmaccountname, &sdr,
										message_ptr);
								break;

							default:
								DO_NOTHING;
								break;
						}
					}
					if (!insert_record && tmp_id[0])
						RESET_MESSAGE("SDR Updated");
				}
				cursor_mgmt(XtParent(w), FALSE);	/* reset it back */
			}
			else
			{
				BEEP_MESSAGE("ID and Title fields are required");
			}
			XtFree(this_title);
			XtFree(this_id);
			break;


		case k_abortpushButton:
			cursor_mgmt(XtParent(w), TRUE);
			/* Get value of k_idtextField into sdr.id[0] */
			last_id = XmTextFieldGetString( Widget_list[ k_idtextField ] );

			/* If it's NEW, use pid instead */
			if (!last_id || emptyline(last_id) || strstr(last_id,"NEW"))
				sprintf(sdr.id[0],"%d",getpid());
			else
			{
				strncpy(sdr.id[0],last_id,sizeof(ID_CODE_T));
				sdr.id[0][sizeof(ID_CODE_T)-1] = '\0';
			}

			/* Check & delete any files found */
			CHECK_AND_DELETE(Tempdir,sdr.id[0],DESCRIPTION_EXT);
			CHECK_AND_DELETE(Tempdir,sdr.id[0],BUILD_EXT);
			CHECK_AND_DELETE(Tempdir,sdr.id[0],FILES_EXT);
			CHECK_AND_DELETE(Tempdir,sdr.id[0],COMMENTS_EXT);
			CHECK_AND_DELETE(Tempdir,sdr.id[0],PROBLEM_EXT);
			CHECK_AND_DELETE(Tempdir,sdr.id[0],RECREATE_EXT);
			CHECK_AND_DELETE(Tempdir,sdr.id[0],RESOLUTION_EXT);
			CHECK_AND_DELETE(Tempdir,sdr.id[0],SOLUTION_EXT);
			CHECK_AND_DELETE(Tempdir,sdr.id[0],TESTING_EXT);
			CHECK_AND_DELETE(Tempdir,sdr.id[0],VALIDATED_EXT);
			CHECK_AND_DELETE(Tempdir,sdr.id[0],PERFORMED_EXT);

			if (!strncmp(sdr.id[0],last_id,sizeof(ID_CODE_T)))
			{
				sprintf(sdr.id[0],"%d",getpid());
				/* Check & delete any "pid" files too */
				CHECK_AND_DELETE(Tempdir,sdr.id[0],DESCRIPTION_EXT);
				CHECK_AND_DELETE(Tempdir,sdr.id[0],BUILD_EXT);
				CHECK_AND_DELETE(Tempdir,sdr.id[0],FILES_EXT);
				CHECK_AND_DELETE(Tempdir,sdr.id[0],COMMENTS_EXT);
				CHECK_AND_DELETE(Tempdir,sdr.id[0],PROBLEM_EXT);
				CHECK_AND_DELETE(Tempdir,sdr.id[0],RECREATE_EXT);
				CHECK_AND_DELETE(Tempdir,sdr.id[0],RESOLUTION_EXT);
				CHECK_AND_DELETE(Tempdir,sdr.id[0],SOLUTION_EXT);
				CHECK_AND_DELETE(Tempdir,sdr.id[0],TESTING_EXT);
				CHECK_AND_DELETE(Tempdir,sdr.id[0],VALIDATED_EXT);
				CHECK_AND_DELETE(Tempdir,sdr.id[0],PERFORMED_EXT);
			}
			RESET_ALLFIELDS();

			/* set id text field "good" marker */
			XtVaSetValues(Widget_list[k_idtextField],
				XmNuserData, False,
				NULL);

			/* adjust sensitivity of scale */
            if (!XtIsSensitive(Widget_list[k_priorityscale]))
                XtSetSensitive(Widget_list[k_priorityscale], True);

			cursor_mgmt(XtParent(w), FALSE);
			break;

		default:
			break;
	}

	/* clear category selection menu */

	return;
}
/* end file : $Source: /home/sydd/sdrtool/RCS/apply_abort_callback.c,v $ */
