#ifndef lint
static char sccsid[] = "$Id: cursor_mgmt.c,v 1.2 1995/01/30 02:36:39 sydd Exp $ SyddWare";
#endif
/*****************************************************************************
* Classification:    Unclassified
*
* Project:           SDRTOOL
*
* Module Name:       cursor_mgmt.c
*
* Description:
*	This routine changes a screen's cursor either to a stop cursor or back
*	to the screen's original cursor.  This is based upon the put_up_stop
*	flag that is passed in as an input argument.  If the original cursor
*	is put back up, all pending X events, except EXPOSE ones, are flushed
*	without being dispatched.  This is so that while the cursor is the
*	stop cursor, the operator may do nothing in the current application.
*
* Calling Sequence:
*	void cursor_mgmt( Widget screen, BOOLEAN put_up_stop) ;
*
*   Inputs:
*		Widget screen - screen whose cursor needs to be changed
*		BOOLEAN put_up_stop - TRUE -> put up stop cursor
*							  FALSE -> put back original cursor
*
*   Outputs:
*		None
*
*   Returns:
*		None
*
* Interprocess Communications
*   Sends:
*       None
*
*   Receives:
*       None
*
* Error Conditions and Actions:
*   N/A
*
* Side Effects:
*	When the stop cursor is up, all subsequent X Events are dropped except
*	for EXPOSE events.
*
* Limitations:
*   N/A
*
* Notes:
*	N/A
*
* Revision History
*   DATE    S/W ENGINEER     DESCRIPTION OF WORK
* --------  ---------------  ------------------------------------------------
* 10/1/93   Tim Arnett       Initial Entry
* 01/06/94	EJ Wassmer		 Prepared for code review
* 01/22/94	EJ Wassmer		 Code review updates
* 04/05/94  J.R.(Sydd)Souza  T00000 - Placed Under CM Control
******************************************************************************/

/*** include files ***/

#include <Mrm/MrmAppl.h>    	/* for Widget       			*/
#include <X11/cursorfont.h>		/* for XC_watch					*/
#include "generi.h"				/* for TRUE & FALSE				*/
#include "generi_t.h"			/* for BOOLEAN					*/


void
cursor_mgmt(
	Widget screen,							/* screen which needs cursor	*/
	BOOLEAN put_up_stop)					/* stop cursor or original		*/
{
	/*** variable declarations ***/

	XtAppContext context ;				/* this applications context		*/
	Cursor cursor ;						/* cursor to create					*/
	XEvent event ;                      /* event to just throw on ground    */


	/*** routine body ***/


	if (put_up_stop)	/* make stop cursor current for this window */
	{
    	cursor = XCreateFontCursor(XtDisplay(screen), XC_watch) ;
    	XDefineCursor(XtDisplay(screen), XtWindow(screen), cursor) ;
    	XFlush(XtDisplay(screen)) ;
	}
	else	/* put original cursor back for this screen */
	{
    	context = XtWidgetToApplicationContext(screen) ;

		/*** discard any X-Events made during the stop time ***/

    	while(XtAppPending(context) != 0)
    	{
        	XtAppNextEvent(context, &event) ;

			/*** "DO" allow expose events to be processed ***/

			if (event.type == Expose)
				XtDispatchEvent(&event) ;
           /*
           ** need to check for PropertyNotify events too
           ** handle SHUTDOWN messages
           */
           if(event.type == PropertyNotify)
           {
              /*
              ** if (notified of shutdown)
              **    rval = SHUTDOWN;
              */
              XtDispatchEvent(&event) ;
			}
		}

    	XUndefineCursor(XtDisplay(screen), XtWindow(screen)) ;
    }

	return ;

}	/* end cursor_mgmt */
/* end file : cursor_mgmt.c */
