#ifndef	lint
static char sccsid[] = "@(#)sqlprintf.c	1.2 4/20/94 SyddWare";
#endif
/*****************************************************************************
 * Classification:  Unclassified
 *
 * Project:         SDRTOOL
 *
 * Module Name:     sqlprintf.c
 *
 * Description:     This file provides the sqlprintf() function which is like
 *                  sprintf() except that there are two additional format
 *                  specifiers: ~d and ~s to handle integers and string fields
 *                  for SQL.  These format specifiers will put strings in
 *                  single quotes and replace empty strings and zero integers
 *                  with the "null" string.
 *
 * Arguments:       s      Returns the string.
 *                  format Specifies the format.
 *                  ...    Specifies the arguments.
 *
 * Returns:         The number of characters in s.
 *
 * Module History:  F. Hellwig  3/25/92  Created.
 *                  J. R. Souza 8/27/92 Added non-ANSI c Sun compatibilities
 *                                      Also changed search_and_replace routine
 *                                      to SEARCH_AND_REPLACE macro
 *****************************************************************************/
#include	"generi_.h"

#ifndef	__STDC__
#	define	memmove(_to,_from,_length) bcopy(_from, _to, _length)
#	ifdef	__sun
#		include <varargs.h>
#	else
#		include <stdarg.h>
#	endif
#else
#	include <varargs.h>
#	include <string.h>
#endif   /* __STDC__ */

#define	SEARCH_AND_REPLACE(_dest, _from, _to) \
{\
    char *__ptr = _dest;\
    while (__ptr = strstr(_dest, _from))\
    {\
		memmove(__ptr + strlen(_to),\
			__ptr + strlen(_from),\
			strlen(__ptr) - strlen(_from) + 1);\
		strncpy(__ptr, _to, strlen(_to));\
    }\
}

#ifndef	__STDC__
int sqlprintf(s, format, va_alist)
char *s;
char *format;
va_dcl
#else
int sqlprintf(char *s, char *format, ...)
#endif
{
    va_list ap;			/* argument pointer */
    char *local_format;		/* local format string */

#ifndef	__STDC__
    va_start(ap);
#else
    va_start(ap,format);
#endif

    /*
     * Allocate memory for local format string.  Double the size of the
     * original format string to allow for expansion and add one for null.
     */

	if (!(local_format = (char *)MALLOC(5 * strlen(format) + 1)))
		return 0;

    strcpy(local_format, format);
    
    /*
     * Replace ~s with ~'%s' and ~d with ~%d.
     */

    SEARCH_AND_REPLACE(local_format, "~d", "~%d");
    SEARCH_AND_REPLACE(local_format, "~s", "~'%s'");
    
    /*
     * Print using local format string.
     */
    
    vsprintf(s, local_format, ap);

    /*
     * Replace ~'' and ~0 with "null" and remove any stray ~.
     */

    SEARCH_AND_REPLACE(s, "~''", "null");
    SEARCH_AND_REPLACE(s, "~0", "null");
    SEARCH_AND_REPLACE(s, "-1", "null");
    SEARCH_AND_REPLACE(s, "~", "");

    FREE(local_format);
    va_end(ap);			/* cleanup */

    return strlen(s);
}
