/*****************************************************************************
* @(#)generi.h	1.3 3/14/94 SyddWare
*
* Classification:  Unclassified
*
* Project:         SDRTOOL
*
* Binding Name:    generi.h
*
* Description:
*   Contains generic constants & macros for SDRTOOL not specific to any aspect 
*	of the project.
*
* Notes:
*	N/A
*
* Revision History
*   DATE    S/W ENGINEER     DESCRIPTION OF WORK
* --------  ---------------  ------------------------------------------------
* 04/01/93  J.R.(Sydd)Souza  Created
******************************************************************************/
#ifndef	GENERI_H
#	define	GENERI_H

/* Note: __STDC__ will be defined if compiler conforms to standard ANSI C */
/*       __CLCC__ will be provided if the compiler is CenterLine's clcc */

/*
** Include Files:
*/

#	include <stdio.h>			/* for FILE type & routines */
#	include <errno.h>			/* for errno.h */
#	include <string.h>			/* for strrchr() */
#	include <unistd.h>			/* for access(), etc. prototypes */

#	include "generi_t.h"	/* for FILEHANDL */

/*
**	Define constants
*/
#	ifndef	FALSE
#		define	FALSE	0
#	endif	/* !FALSE */
#	define	BITWISE_FALSE	0	/* for use in bitfields */

#	ifndef	TRUE
#		define	TRUE	!FALSE
#	endif	/* !TRUE */
#	define	BITWISE_TRUE	1	/* for use in bitfields */

/* typed nulls for aiding readability */
#	ifndef	NULL
#		define	NULL	0
#	endif	/* !NULL */
#	define	NULLC	('\0')
#	define	NULLSTR	((char *)"")
#	define	NULLI	((int) 0)
#	define	NULLS	((short) 0)
#	define	NULLL	((long) 0)
#	define NULLP	((char *) 0)
#	define	NULLFI	((FILE *) 0)
#	define	NULLFH	((FILEHANDL) 0)
#	define	NULLFL	((float) 0)
#	define	NULLD	((double) 0)

#	define	ERROR			-1
#	define SUCCESS			 0

/*
**	DO_NOTHING
**	Macro for use at the end of for loops ow whiles that have no body.
**	Makes code a tad more readable and prevents misunderstanding due to
**	missed semicolon.  Example usage before (mystr is a null terminated string):
**
**	for (icnt=0; mystr[icnt] != 0 && mystr[icnt] != '.'; ++icnt);
**
**	The above is supposed to walk down mystr looking for the first period
**	or the end of string, whichever comes first (never mind that you can
**  use strchr() to do essentially the same thing).  It's easy to miss the
**  semicolon at the end of the line and a reviewer or maintenance person
**  might assume the line following the for loop has something to do with it.
**	Instead use the following to make it clear that the for loop is self
**	contained:
**
**	for (icnt=0; mystr[icnt] != 0 && mystr[icnt] != '.'; ++icnt)
**		DO_NOTHING;
**
**  This leaves little doubt.
*/
#	define	DO_NOTHING

/* The following are for process termination return values */
#	define SUCCESSFUL_SHUTDOWN   0
#	define FILE_OPEN_FAILURE 	-10
#	define COMMS_INIT_FAILURE 	-20
#	define CONFIG_LOAD_FAILURE -30
#	define DB_CONNECT_FAILURE  -40
#	define GETENV_FAILURE      -50
#	define COMMS_RECEIVE_FAILURE -70

#	define MAX_FILENAME_SIZE		(255)	/* general, not incl. null */
#	define MAX_PROCESS_NAME_LENGTH	(40)

#	define PUBLIC
#	define PRIVATE     static

/* defines for send_data and Ccb_send_alarm but generic enough for here */

#	define WAIT 0
#	define NO_WAIT 1

#	define ACK 1
#	define NO_ACK 0

/*
**  Parms for access() call to make logic a little easier to read:
**  access() returns FALSE if test is successful, so that access(filename,0)
**  which asks if a file exists, returns FALSE meaning it does exist, not-FALSE
**  if it does not exist.  Using the below values, you can more easily
**  understand the following if construct:
**      if (access(filename, NOTEXIST)) { ... }
**  which, if FALSE is returned (filename exists) the object of the if is not
**  performed, and if not-FALSE is returned (file does not exist), the if object
**  is performed.
*/
#	define NOTEXIST        0   /* returns FALSE if file exists */
#	define NOTWRITABLE     2   /* returns FALSE if write access is allowed */
#	define NOTREADABLE     4   /* returns FALSE if read access is allowed */
#	define NOTREADWRITE    6   /* returns FALSE if read/write access allowed */

/* mask for UNIX open() */
#	ifndef O_RAW
#	   define  O_RAW   0
#	endif /* O_RAW not defined */

/*
**	JUST_NAME()
**	Since we are using __FILE__ for function name, but the compiler
**	seems to be placing the pasth of the source file in (like
**	/devel/c/c2/c22/yadayadayada.c, use this macro to get just the
**	filename.  Call it thusly:
**	JUST_NAME(__FILE__);
*/
#	define	JUST_NAME(_ptr)	(char *)((strrchr((_ptr),'/'))?\
	strrchr((_ptr),'/')+1:(_ptr))

/*
**	Prototyping fakout code:
**		if __STDC__ is defined, PROTO(a) is replaced by a
**		if __STDC__ is not defined, PROTO(a) is replaced by ()
**	The former is used when the compiler is ANSI C compatible (compiler SHOULD
**	supply the __STDC__ constant, per the standard), the latter in non-ANSI C
**	environments (such as Unix).
**
**	In any environment, routine prototypes should be defined as follows:
**
**		int bob PROTO(( char sam, int joe ));
**
**	which, in ANSI C environment, becomes int bob ( char sam, int joe ); and, in
**	non-ANSI C environment, becomes int bob (); [note that double parentheses
**	are required above].
**	Note: It seems CenterLine's (__CLCC__) compiler, in -Xa mode,
**	also provide prototype checking, so I enabled PROTO() for it too.
*/
#	if defined(__STDC__) || defined(__CLCC__)
#		ifndef PROTO
#			define PROTO(_a)	_a
#		endif /* PROTO not defined */
#	else /* __STDC__ && __CLCC__ not defined */
#		ifndef PROTO
#			define	PROTO(_a)	()
#		endif /* PROTO not defined */
#	endif /* __STDC__ || __CLCC__ */

#	ifdef	MAIN_PROGRAM
#		ifdef	EXTERN
#			undef	EXTERN
#		endif	/* EXTERN */
#		define	EXTERN
#		ifdef	INIT
#			undef	INIT(_a)
#		endif	/* INIT(_a) */
#		define	INIT(_a)	={_a}
#	else	/* !MAIN_PROGRAM */
#		ifdef	EXTERN
#			undef	EXTERN
#		endif	/* EXTERN */
#		define	EXTERN	extern
#		ifdef	INIT
#			undef	INIT(_a)
#		endif	/* INIT(_a) */
#		define	INIT(_a)
#	endif	/* MAIN_PROGRAM */


/*
**	Debugging aids: allow you to turn on or off DEBUG code via one
**	C compiler command line option, all based around the constant
**	DEBUG (sometimes supplied by the compiler).
**	NOTE: Both ANSI-C (token pasting) and K&R versions are included.
*/
#	ifdef DEBUG

extern	char	*sys_errlist[]; /* provided by O/S */
/*
**	External def of a global FILE struct pointer Out_strm defined in out_strm.c
**	[which is compiled and added to libsydd lib (thus included at link time only
**	if needed)].  This FILE pointer is predefined to a null pointer so if it is
**	never opened (via DEBUGOPEN macro), DIAG, DIAGn and DUMP macros will not
**	write to it.
*/
EXTERN	FILE	*Out_strm
#		ifdef	MAIN_PROGRAM
= {stderr}
#		endif	/* MAIN_PROGRAM */
;	/* see also Out_strm.c file in libcommon */

#		ifdef __STDC__	/* use ANSI string replacement format pound sign prefix */

/*
**	DUMP() macro -
**	dump.c dumps size bytes from address defined by buf to file specified
*/
#			define	DUMP(_buf,_size)	\
if (Out_strm)\
{\
	fprintf(Out_strm,"Dump of "#_size" bytes of "#_buf",e=%d[%s]\n",\
		errno,sys_errlist[errno]);\
	dump((unsigned char *)(_buf),(int)(_size),(FILE *)Out_strm);\
	fflush(Out_strm);\
}

/*
**	DEBUGOPEN() macro -
**		Opens filename specified [not in quotes, such as DEBUGOPEN(myfile.log);
**		for write access and assigns FILE pointer returned to global Out_strm,
**		to be used by any DIAG, DIAGn and DUMP statements.
**		Note if fopen() fails for some reason, Out_strm is set to stderr.
**		Also note: stderr is redirected to this file as well in case the O/S or
**		some standard C function writes there too; this allows these messages
**		to be saved and in sequence with other debug code.
*/
#			define DEBUGOPEN(_filename) \
{\
    if (access(#_filename,NOTEXIST)||\
        !access(#_filename,NOTWRITABLE))\
    {\
        if (!(Out_strm=freopen(#_filename,"w",stderr)))\
        {\
            /* WARNING: stderr is now closed regardless */\
            printf("DEBUGOPEN: *WARNING* attempt to open %s failed\n",\
				#_filename);\
            printf("DEBUGOPEN: *WARNING* failure caused close of stderr\n");\
            printf("DEBUGOPEN: *WARNING* stderr is no longer available\n");\
            printf("DEBUGOPEN: *WARNING* DEBUG data redirected to stdout\n");\
            Out_strm=stdout;\
        }\
		else\
			errno = 0; /* all is well, reset errno from access() */\
    }\
    else /* all else fails */\
        Out_strm=stderr;\
}

/*
**	DEBUGCLOSE macro -
**		Closes Out_strm file, if open, when done - note that this macro takes no
**		arguments, that Out_strm is only closed if it is defined and not
**		equivalent to stderr, and that Out_strm is reset to a null FILE pointer
**		to prevent attempts to write to it after the close.
*/
#			define	DEBUGCLOSE		\
if 	(Out_strm&&Out_strm!=stderr)\
{\
	fclose(Out_strm);\
	Out_strm=NULLFI;\
}

/*
**	DIAG() macro -
**		Print text in parentheses to Out_strm; stream is flushed after each
**		write to ensure that all writes get to the file (useful is program
**		crashes, as buffered I/O does not guarantee that all writes are
**		actually done until stream is closed or flushed).
**
**		e.g. DIAG(hello my name is sydd);
**		sends the following line to Out_strm:
**
**		hello my name is sydd,e=#[msg]
**	
**		the e=# represents the current value of the C global variable errno
**		(# will be the value of errno at the time of the call.  This is also
**		useful for seeing failures of routines which set this variable.
**      the [msg] is the value of sys_errlist[errno] at the time of the call
**		Most systems provide [Error 0] when errno is 0, otherwise, the
**		message will be quite useful.  Remember though that errno is never
**		usually reset to 0 so set it to 0 before a call that might set it
**		on error to ensure that it was set by the routine in question.
*/
#			define	DIAG(_str1)	\
if (Out_strm)\
{\
	fprintf(Out_strm,#_str1",e=%d[%s]\n",\
		errno,sys_errlist[errno]);\
	fflush(Out_strm);\
}

/*
**	DIAG1() macro -
**		Similar to DIAG(), but taking a string, a variable (or constant) and a
**		printf() type specification.
**
**		e.g. DIAG1(current value of,temp,s);	[assume temp is char temp[] =
**				"hello";]
**		produces the following line in Out_strm:
**
**		current_value of temp = <hello>,e=#[msg]
**
**		Note that the 2nd parm can be anything, a variable, a string constant,
**		etc, but that the 3rd parm [a printf() family type code spec such as
**		what would follow the percent (%) character] must be compatible.
**		Also, the 3rd parm can be any valid % spec such as 02.2d lf 10.1s etc.
*/
#			define	DIAG1(_str1,_arg1,_typ1)	\
if (Out_strm)\
{\
	fprintf(Out_strm,\
		#_str1" "#_arg1"=<%"#_typ1">,e=%d[%s]\n",\
		_arg1,errno,sys_errlist[errno]);\
	fflush(Out_strm);\
}

/*
**	DIAG2() macro -
**		Same as DIAG1() except it takes 2 groups of string,value,printype.
**
**	e.g. DIAG2(hello my name is,sydd,s,and my dog is,years_old,d);
**		[assume char sydd[] = "Sydd"; int years_old = 10;]
**	produces the following line in Out_strm:
**
**	hello my name is sydd = <Sydd> and my dog is years_old = <10>,e=#[msg]
*/
#			define	DIAG2(_str1,_arg1,_typ1,_str2,_arg2,_typ2)	\
if (Out_strm)\
{\
	fprintf(Out_strm,\
		#_str1" "#_arg1"=<%"#_typ1"> "\
		#_str2" "#_arg2"=<%"#_typ2">,e=%d[%s]\n",\
		_arg1,_arg2,errno,sys_errlist[errno]);\
	fflush(Out_strm);\
}

/*
**	DIAG3() macro -
**		Same as DIAG2() except it takes 3 groups of string,value,printype.
**
**	e.g. DIAG2(hello my name is,sydd,s,and my dog is,years_old,d,and he
**			is,really,s);
**	  [assume char sydd[] = "Sydd"; int years_old = 10; char really[] = "ugly";]
**	produces the following line in Out_strm:
**
**	hello my name is sydd = <Sydd> and my dog is years_old = <10> and he is
**	really = <ugly>,e=#[msg]
*/
#			define	DIAG3(_str1,_arg1,_typ1,_str2,_arg2,_typ2,_str3,_arg3,_typ3) \
if (Out_strm)\
{\
	fprintf(Out_strm,\
		#_str1" "#_arg1"=<%"#_typ1"> "\
		#_str2" "#_arg2"=<%"#_typ2"> "\
		#_str3" "#_arg3"=<%"#_typ3">,e=%d[%s]\n",\
		_arg1,_arg2,_arg3,errno,sys_errlist[errno]);\
	fflush(Out_strm);\
}
#		endif /* __STDC__ */

/*
**	For non-ANSI C standard environments, replacement of strings in macros
**	is handled slightly differently (as in UNIX).  Arguments destined for
**	verbatim output need simply to be placed inside the quotes and they
**	are literally printed by name.
*/
#		ifndef __STDC__
/*
**	DUMP() macro -
**	dump.c dumps size bytes from address defined by buf to file specified
*/
#			define	DUMP(_buf,_size)	\
if (Out_strm)\
{\
	fprintf(Out_strm,"Dump of _size bytes of _buf,e=%d[%s]\n",\
		errno,sys_errlist[errno]);\
	dump((unsigned char *)(_buf),(int)(_size),(FILE *)Out_strm);\
	fflush(Out_strm);\
}

/*
**	DEBUGOPEN() macro - See description above
*/
#			define DEBUGOPEN(_filename) \
{\
    if (access("_filename",NOTEXIST)||\
        !access("_filename",NOTWRITABLE))\
    {\
        if (!(Out_strm=freopen("_filename","w",stderr)))\
        {\
            /* WARNING: stderr is now closed regardless */\
            printf("DEBUGOPEN: *WARNING* attempt to open %s failed\n",\
				"_filename");\
            printf("DEBUGOPEN: *WARNING* failure caused close of stderr\n");\
            printf("DEBUGOPEN: *WARNING* stderr is no longer available\n");\
            printf("DEBUGOPEN: *WARNING* DEBUG data redirected to stdout\n");\
            Out_strm=stdout;\
        }\
		else\
			errno = 0; /* all is well, reset errno from access() */\
    }\
    else /* all else fails */\
        Out_strm=stderr;\
}

/*
**	DEBUGCLOSE() macro - See description above
*/
#			define	DEBUGCLOSE		\
if (Out_strm&&Out_strm!=stderr)\
{\
	fclose(Out_strm);\
	Out_strm=NULLFI;\
}

/*
**	DIAG() macro - See description above
*/
#			define	DIAG(_str1)	\
if (Out_strm)\
{\
	fprintf(Out_strm,"_str1,e=%d[%s]\n",errno,sys_errlist[errno]);\
	fflush(Out_strm);\
}

/*
**	DIAG1() macro - See description above
*/
#			define	DIAG1(_str1,_arg1,_typ1)	\
if (Out_strm)\
{\
	fprintf(Out_strm,\
		"_str1 _arg1 = <%_typ1>,e=%d[%s]\n",\
		_arg1,errno,sys_errlist[errno]);\
	fflush(Out_strm);\
}

/*
**	DIAG2() macro - See description above
*/
#			define	DIAG2(_str1,_arg1,_typ1,_str2,_arg2,_typ2)	\
if (Out_strm)\
{\
	fprintf(Out_strm,\
		"_str1 _arg1 = <%_typ1> _str2 _arg2 = <%_typ2>,e=%d[%s]\n",\
		_arg1,_arg2,errno,sys_errlist[errno]);\
	fflush(Out_strm);\
}

/*
**	DIAG3() macro - See description above
*/
#			define	DIAG3(_str1,_arg1,_typ1,_str2,_arg2,_typ2,_str3,_arg3,_typ3) \
if (Out_strm)\
{\
	fprintf(Out_strm,\
"_str1 _arg1 = <%_typ1>\
 _str2 _arg2 = <%_typ2>\
 _str3 _arg3 = <%_typ3>,e=%d[%s]\n",\
		_arg1,_arg2,_arg3,errno,sys_errlist[errno]);\
	fflush(Out_strm);\
}

#		endif /* __STDC__ */

#	else /* DEBUG not defined */
/*
**	Define all macros to nothing when DEBUG is not defined, thereby
**	excluding all references from compile.
*/
#		define	DEBUGOPEN(_filename)
#		define	DEBUGCLOSE
#		define	DIAG(_str1)
#		define	DIAG1(_str1,_arg1,_typ1)
#		define	DIAG2(_str1,_arg1,_typ1,_str2,_arg2,_typ2)
#		define	DIAG3(_str1,_arg1,_typ1,_str2,_arg2,_typ2,_str3,_arg3,_typ3)
#		define	DUMP(_buff,_size)

#	endif /* DEBUG */

/*
**  Defines the time between attempts at database login 
**  and the maximum number of attempts 
*/
#	ifndef	DBLOGIN
#		define LOGIN_RETRY_DELAY      "LOGIN_RETRY_DELAY"
#		define MAX_DB_LOGIN_ATTEMPTS  "MAX_DB_LOGIN_ATTEMPTS"
#	endif	/* DBLOGIN */

#endif /* GENERI_H no source lines of code after this line */
/* end file : generi.h */
