/*****************************************************************************
* $Id: sdr_t.h,v 1.3 1995/01/30 01:33:28 sydd Exp $ SyddWare
*
* Classification:  Unclassified
*
* Project:         SDRTOOL
*
* Binding Name:    sdr_t.h
*
* Description:
*   Contains typedefs for SDRTool
*
* Notes:
*   N/A
*
* Revision History
*   DATE    S/W ENGINEER     DESCRIPTION OF WORK
* --------  ---------------  ------------------------------------------------
* 04/01/93  J.R.(Sydd)Souza  Created
******************************************************************************/
#ifndef	SDR_T_H
#	define SDR_T_H

#	include <sys/types.h>	/* for pid_t */
#	include "sdr.h"		/* for ID_CODE_SIZE etc. */

/* typedef	int		BOOLEAN; */
typedef char	ID_CODE_T[ID_CODE_SIZE +1];
typedef	char	DATE_STRING_T[DATE_STRING_SIZE +1];

typedef	enum
{	/* at AST use tolower() on these first */
	DOCUMENTATION_PREFIX = 'D',
	HARDWARE_PREFIX = 'H',
	ORIGINAL_PREFIX = 'N',
	SOFTWARE_PREFIX = 'S',
	TOOL_PREFIX = 'T',
	VENDOR_PREFIX = 'V'
} ID_PREFIX_T;

typedef	enum
{
	NEW_CCB1 = 1,
	ASSESS,
	ASSIGNED,
	CCB2,
	RESOLVED,
	REJECTED,
	CMEVAL,
	PROMOTE
} STATUS_T;

typedef	struct	SDR_STRUCT
{
							/* time_to_complete */
	unsigned long	hours;	/* stored as hundredths of hours: 1 hour=100 */
	STATUS_T		status;
	ID_CODE_T		id[2];	/* normal_id = 0 - assigned_id = 1 */
	DATE_STRING_T	originated;	/* origin_date */
	DATE_STRING_T	assigned; /* assign_date */
	DATE_STRING_T	due;	/* due_date */
	DATE_STRING_T	closed;	/* close_date */
	DATE_STRING_T	transition;	/* transition_date */
	char			title[TITLE_SIZE +1];
	char			originator[ORIGINATOR_SIZE +1];
	char			trouble_area[TROUBLE_AREA_SIZE +1];
	char			assigned_to[ASSIGNED_SIZE +1];
	char			cpc_mnemonic[CPC_CODE_SIZE +1]; /* cpc */
	char			thread[THREAD_SIZE +1];
	unsigned char	priority;	/* 1-5 */
} SDR_T;

typedef	struct  PROCESS_TABLE_STRUCT
{
    char    	*extension; /* will match one of the known ones named *_EXT */
    pid_t   	pid;
	ID_CODE_T	id;
} PROCESS_TABLE_T;

typedef	struct	SDRRESOURCES
{
    char    *sdrpath;       /* SDRPATH default */
    char    *sdridfile;     /* ".assign" default */
    char    *sdrsqlcmd; /* SDRSQLCMD default */
    char    *cmaccountname; /* CM_ACCOUNTNAME default */
    char    *ccbmembers;    /* CCB_MEMBERS default */
} AppData, *AppDataPtr;

typedef struct  CPC_NAME_STRUCT
{
    char *mnemonic;
    char *name;
    Widget  widgee;
} CPC_NAME_T;

typedef struct  VENDOR_NAME_STRUCT
{
    char *name;
    Widget  widgee;
} VENDOR_NAME_T;

typedef struct  DOCUMENT_NAME_STRUCT
{
    char *name;
    Widget  widgee;
} DOCUMENT_NAME_T;

typedef struct  HARDWARE_NAME_STRUCT
{
    char *name;
    Widget  widgee;
} HARDWARE_NAME_T;

typedef	struct  SUPPORT_STRUCT
{
    char            *dbname; /* less _sdrs */
    char            *uidfilename; /* full name with .uid */
    CPC_NAME_T      *CPC_table;
    VENDOR_NAME_T   *vendor_table;
    DOCUMENT_NAME_T *document_table;
    HARDWARE_NAME_T *hardware_table;
} SUPPORT_T;

#endif	/* SDR_T_H */
/* end file : sdr_t.h */
