#ifndef	lint
static char rcsid[] = "$Id: insert_discrep_rec.c,v 1.5 1995/03/20 03:08:41 sydd Exp $ SyddWare";
#endif
/*****************************************************************************
* Classification:    UNCLASSIFIED
*
* Project:           SDRTOOL
*
* Module Name:       insert_discrep_rec.c 
*
* Description:       
*   insert_discrep_rec inserts a row into the DISCREP_REP table.
*
* Calling Sequence:  
*   int status = insert_discrep_rec      
*                        ( SDR_T  *discrep_record );
*
*   Inputs:
*       discrep_record            - pointer to a structure containing the 
*                                   record to be inserted.
*
*   Outputs: 
*     None
*
*   Returns:
*       status - returns the contents of the dbretstat member 
*                of the dbproc structure.  
*                If the record was successfully inserted dbretstat = SUCCESS.
*                If the record was not successfully inserted 
*                dbretstat != SUCCESS and the database administrator will have 
*                to examine the code to determine the cause of the failure.
*
* Error Conditions and Actions:
*   if sqlprintf, or dbcmd, or dbsqlexec fail,
*   an alarm is sent and ERROR is returned to the calling program.
*
* Side Effects:      
*
*   If successful, inserts one row in the DISCREP_REP table.
*
* Limitations: 
*   The application program must call login_to_database, to obtain
*   a DBPROCESS structure, before calling this function. 
*
* Notes: 
*   N/A
*
* Revision History
*   DATE    S/W ENGINEER     DESCRIPTION OF WORK
* --------  ---------------  ------------------------------------------------
* 09-22-93  J.R.(Sydd)Souza  Initial Creation of C Routine
* 03/22/94  J.R.(Sydd)Souza  Removed sybase specifics
* 01/11/95  J.R.(Sydd)Souza  Informix variation mods
******************************************************************************/

/*
** Include Files
*/
#include <stdlib.h>			/* Prototype for system()                 */
#include "sdr_.h"		/* for this file's prototype	*/

/*
** Function get_discrep_rec
*/

int insert_discrep_rec (discrep_record)
	SDR_T *discrep_record;
{
    /* 
    **  Variable Definitions and Declarations
    **
    **************************************************************************
    **************************************************************************
    **   sql_string             - string containing stored procedure exec   **
    **                            command                                   ** 
    **************************************************************************
    */

	FILE				*tempfile = (FILE *)0;
	int					 returnval = SUCCESS;
	int					 system_results = 0;
	char        		 sql_string[3*MAX_SQL_STRING_LEN];          
	char        		 system_command[MAX_SQL_STRING_LEN];          
	char        		 command_filename[L_tmpnam];          
	char        		 script_filename[L_tmpnam];          
	char        		 results_filename[L_tmpnam];          

    if (!discrep_record)
    {
        return(ERROR);
    }

    /*
    ** Build and execute the SQL command to insert into the  **
    ** DISCREP_REP table using the DODBCMD macro             **
    */
    sqlprintf(sql_string,"\
insert into %s_sdrs\n\
(NORMAL_ID, ASSIGNED_ID, TITLE, STATUS, PRIORITY, ORIGIN_DATE, ORIGINATOR,\n\
 ASSIGN_DATE, ASSIGNED_TO, DUE_DATE, CLOSE_DATE, CPC, THREAD,\n\
 TIME_TO_COMPLETE, TROUBLE_AREA, TRANSITION_DATE, CO_ASSIGNEE)\n\
values (~s, ~s, ~s, %hd, %hd, ~s, ~s, ~s, ~s, ~s, ~s, ~s, ~s, %d, ~s, ~s, ~s)\n",
		Table_name,
        discrep_record->id[0],
		discrep_record->id[1],
		discrep_record->title,
		discrep_record->status,
		discrep_record->priority,
		discrep_record->originated,
		discrep_record->originator,
		discrep_record->assigned,
		discrep_record->assigned_to,
		discrep_record->due,
		discrep_record->closed,
		discrep_record->cpc_mnemonic,
		discrep_record->thread,
		discrep_record->hours,
		discrep_record->trouble_area,
		discrep_record->transition,
		discrep_record->co_assignee);
	command_filename[0] = script_filename[0] = results_filename[0] = '\0';
	if (!tmpnam(command_filename) ||
		!tmpnam(script_filename) ||
		!tmpnam(results_filename) ||
		!(tempfile = fopen(command_filename,"w")) ||
		!fprintf(tempfile,"%s\n", sql_string) ||
		fclose(tempfile) ||
		(tempfile = (FILE *)0) ||
		!sprintf(system_command,"%s %s <%s >%s 2>&1",
			SdrTool.sdrsqlcmd, SDRTOOL_DB_NAME, command_filename,
			results_filename) ||
		!(tempfile = fopen(script_filename,"w")) ||
		!fprintf(tempfile,"#!/bin/sh\n%s\n", system_command) ||
		fclose(tempfile) ||
		(tempfile = (FILE *)0) ||
		!sprintf(system_command,"/bin/sh %s", script_filename))
	{
		returnval = ERROR;
	}
	else
	{
		system_results = system(system_command);
		if (access(results_filename, NOTEXIST) ||
			!(tempfile = fopen(results_filename,"r")))
		{
			returnval = ERROR;
		}
		else
		{
			returnval = ERROR;
			while (returnval == ERROR &&
				fgets(sql_string,sizeof(sql_string),tempfile))
				if (strstr(sql_string,"1 row(s) inserted"))
					returnval = SUCCESS;
		}
	}
	if (tempfile)
	{
		fclose(tempfile);
		tempfile = (FILE *)0;
	}
#define DELETE_THEM
#ifdef DELETE_THEM
	if (!emptyline(command_filename) &&
		!access(command_filename,NOTEXIST))
		remove(command_filename);
	if (!emptyline(script_filename) &&
		!access(script_filename,NOTEXIST))
		remove(script_filename);
	if (!emptyline(results_filename) &&
		!access(results_filename,NOTEXIST))
		remove(results_filename);
#else  /* DELETE_THEM */
	printf("%s\n\
command_filename [%s]\n\
script_filename  [%s]\n\
results_filename [%s]\n",
		__FILE__, command_filename, script_filename, results_filename);
#endif /* DELETE_THEM */
    return (returnval);
} /* end function: get_discrep_rec */  

/* end file : insert_discrep_rec.c */ 
