#ifndef	lint
static char sccsid[] = "W% 8/31/95 SyddWare";
#endif
/****************************************************************************
* Classification:    Unclassified
*
* Project:           SDRTool
*
* Module Name:       next_normal_id.c
*
* Description:
*   This standalone executable can be used to get the next "N" number from
* a given sdridfile (file name & path must be supplied).  This is to be
* used to test the lockout_idfile(), unlock_idfile() and get_next_id()
* routines and can be used in scripts outside the SDRTool to generate
* SDRs automagically.
*
* Calling Sequence:
*   next_normal_id sdridfile
*
*   Inputs:
*       sdridfile - typically the .assign file but may be specified
*                   differently in the .sdrIdFile resource for a given
*                   project in the SdrTool app-defaults file.  The name
*                   specified here must be the full pathname of the file
*                   unless you're running this from the directory in which
*                   the file exists.  In app-defaults resource terms:
*                     sdrPath + "/" + sdrIdFile
*             
*
*   Outputs:
*       The "N" number to ne used for an SDR.  Note that this number is
*       now registerred as used, so there's no going back - no un-using it.
*
*   Returns:
*       None
*
* Interprocess Communications
*	Sends:
*       None
*
*   Receives:
*       None
*
* Error Conditions and Actions:
*   N/A
*
* Side Effects:
*   N/A
*
* Limitations:
*   N/A
*
* Notes:
*   N/A
*
* Revision History
*   DATE    S/W ENGINEER     DESCRIPTION OF WORK
* --------  ---------------  ------------------------------------------------
* 08/31/95  J.R.(Sydd)Souza  Created
******************************************************************************/
/*
 * System Include Files
 */
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/stat.h>   /* for S_IRWXU etc */

#define MAIN_PROGRAM
#include "sdrtool_.h"

AppData SdrTool;

/*
**  ANSI strerror() routine as a UNIX local macro.  We want to use the ANSI
**  one but we mostly use it for UNIX routines.  The UNIX libc.a does not
**  have a strerror() routine and the ANSI strerror() in the CenterLine or
**  Mercury libraries do not support the UNIX routines' error codes beyond
**  those that the UNIX routines use from the standard list.
*/
extern  int sys_nerr;           /* number of errors in sys_errlist[] */
extern  char    *sys_errlist[]; /* provides strings for errno values */
#ifndef strerror
#   define  strerror(_num) ((0 <= _num && _num < sys_nerr)? \
    (char *)sys_errlist[_num]:                              \
    (char *)"*Unknown Error Number*")
#endif  /* !strerror */

/*
 * Main Program.
 */
main(int argc, char **argv)
{
    char        *lockfilename = (char *)0;
    char        *last_slash = (char *)0;
	ID_CODE_T	normal_id;
	int 		save_errno = 0;

	errno = 0;
	if (argc < 2)
	{
		fprintf(stderr,"%s: Error, requires full pathname of sdrIdFile\n",
			argv[0]);
		exit(EINVAL);
	}
	else if (access(argv[1],NOTEXIST))
	{
		save_errno=errno;
		fprintf(stderr,"%s: Error, sdrIdFile [%s]: %s\n",
			argv[0], argv[1], strerror(save_errno));
		exit(save_errno);
	}
	
	memset(&SdrTool,0,sizeof(SdrTool));

	last_slash = strrchr(argv[1],'/');
	if (!last_slash)
	{
		SdrTool.sdrpath = ".";
		SdrTool.sdridfile = argv[1];
	}
	else
	{
		*last_slash = '\0';
		SdrTool.sdrpath = argv[1];
		SdrTool.sdridfile = &last_slash[1];
	}
	if (lockout_idfile(&lockfilename) ||
		get_next_id(ORIGINAL_PREFIX,normal_id,TRUE))
	{
		fprintf(stderr,
			"%s: Error, cannot get next %c-type from sdrIdFile [%s/%s]\n",
			argv[0], ORIGINAL_PREFIX, SdrTool.sdrpath, SdrTool.sdridfile);
		exit(EACCES);
	}
	else
	{
		unlock_idfile(lockfilename);
        lockfilename = (char *)0;
		printf("%s",normal_id); /* note: lack of newline is intentional */
	}
	exit(0);
}
/* end file : next_normal_id.c */
