#ifndef	lint
static char rcsid[] = "$Id: get_discrep_rec.c,v 1.7 1995/05/14 23:55:22 sydd Exp $ SyddWare";
#endif
/*****************************************************************************
* Classification:    UNCLASSIFIED
*
* Project:           SDRTOOL
*
* Module Name:       get_discrep_rec.c
*
* Description:       
*   get_discrep_rec retrieve a row into the DISCREP_REP table.
*
* Calling Sequence:  
*   int status = get_discrep_rec      
*                        ( SDR_T  *discrep_record )
*
*   Inputs:
*       discrep_record            - pointer to a structure containing the 
*                                   record to be inserted.
*
*   Outputs: 
*     None
*
*   Returns:
*       status - returns the contents of the dbretstat member 
*                of the dbproc structure.  
*                If the record was successfully inserted dbretstat = SUCCESS.
*                If the record was not successfully inserted 
*                dbretstat != SUCCESS and the database administrator will have 
*                to examine the code to determine the cause of the failure.
*
* Error Conditions and Actions:
*   if sqlprintf, or dbcmd, or dbsqlexec fail,
*   an alarm is sent and ERROR is returned to the calling program.
*
* Side Effects:      
*
*   If successful, inserts one row in the DISCREP_REP table.
*
* Limitations: 
*   The application program must call login_to_database, to obtain
*   a DBPROCESS structure, before calling this function. 
*
* Notes: 
*   This function invokes the following stored procedures:
*   ins_discrep_rec     - no data from the database is returned.
*
* Revision History
*   DATE    S/W ENGINEER     DESCRIPTION OF WORK
* --------  ---------------  ------------------------------------------------
* 09-22-93  R.K.Montgomery   Initial Creation of C Routine
* 03/22/94  J.R.(Sydd)Souza  Removed sybase specifics
* 01/11/95  J.R.(Sydd)Souza  Informix variation
******************************************************************************/

/*
** Include Files
*/
#include <stdlib.h>			/* Prototype for system()                 */
#include <string.h>         /* Prototype for strstr      */
#include "sdr_.h"			/* for MEMBER_OFFSET() macro */
#include "sdr_g.h"			/* for Table_name */


#define	NOOP_TYPE	0
#define	CHAR_TYPE	1
#define	UCHAR_TYPE	2
#define	SHORT_TYPE	3
#define	INT_TYPE	4
#define	ULONG_TYPE	5
static	struct
	{
		char	*name;
		int		offset;
		int		type;
		int		size;
#ifdef	__STDC__
	} bindings[] =
#else	/* __STDC__ */
	} bindings[18] =
#endif	/* __STDC__ */
{
	{ "normal_id", MEMBER_OFFSET(SDR_T *,id[0]),
		CHAR_TYPE,	ID_CODE_SIZE },
	{ "assigned_id", MEMBER_OFFSET(SDR_T *,id[1]),
		CHAR_TYPE,	ID_CODE_SIZE },
	{ "title", MEMBER_OFFSET(SDR_T *,title),
		CHAR_TYPE,	TITLE_SIZE },
	{ "status", MEMBER_OFFSET(SDR_T *,status),
		INT_TYPE,	sizeof(STATUS_T) },
	{ "priority", MEMBER_OFFSET(SDR_T *,priority),
		UCHAR_TYPE,	sizeof(unsigned char) },
	{ "origin_date", MEMBER_OFFSET(SDR_T *,originated),
		CHAR_TYPE,	DATE_STRING_SIZE },
	{ "originator", MEMBER_OFFSET(SDR_T *,originator),
		CHAR_TYPE,	ORIGINATOR_SIZE },
	{ "assign_date", MEMBER_OFFSET(SDR_T *,assigned),
		CHAR_TYPE,	DATE_STRING_SIZE },
	{ "assigned_to", MEMBER_OFFSET(SDR_T *,assigned_to),
		CHAR_TYPE,	ASSIGNED_SIZE },
	{ "due_date", MEMBER_OFFSET(SDR_T *,due),
		CHAR_TYPE,	DATE_STRING_SIZE },
	{ "close_date", MEMBER_OFFSET(SDR_T *,closed),
		CHAR_TYPE,	DATE_STRING_SIZE },
	{ "cpc", MEMBER_OFFSET(SDR_T *,cpc_mnemonic),
		CHAR_TYPE,	CPC_CODE_SIZE },
	{ "thread", MEMBER_OFFSET(SDR_T *,thread),
		CHAR_TYPE,	THREAD_SIZE },
	{ "time_to_complete", MEMBER_OFFSET(SDR_T *,hours),
		ULONG_TYPE,	sizeof(unsigned long) },
	{ "trouble_area", MEMBER_OFFSET(SDR_T *,trouble_area),
		CHAR_TYPE,	TROUBLE_AREA_SIZE },
	{ "transition_date", MEMBER_OFFSET(SDR_T *,transition),
		CHAR_TYPE,	DATE_STRING_SIZE },
	{ "co_assignee", MEMBER_OFFSET(SDR_T *,co_assignee),
		CHAR_TYPE,	ASSIGNED_SIZE },
	{ "", 0,
		NOOP_TYPE,	0 }
};
/*
** Function get_discrep_rec
*/

int get_discrep_rec (discrep_record)
	SDR_T *discrep_record;
{
    /* 
    **  Variable Definitions and Declarations
    **
    **************************************************************************
    **************************************************************************
    **   sql_string             - string containing stored procedure exec   **
    **                            command                                   ** 
    **************************************************************************
    */

    FILE           *tempfile = (FILE *)0;
    char           *localptr = (char *)0;
    char           *parmptr = (char *)0;
    char           *scanptr = (char *)0;
	unsigned long  *tempulong;
	int			   *tempint;
	short		   *tempshort;
	unsigned int    tmpuint;
	int				bindno = 0;
	int				pos = 0;
	int				linelen = 0;
    int             returnval = SUCCESS;
    int             system_results = 0;
    char            sql_string[MAX_SQL_STRING_LEN];
    char            format_string[1024];
    char            system_command[1024];
    char            command_filename[L_tmpnam];
    char            script_filename[L_tmpnam];
    char            results_filename[L_tmpnam];
	SDR_T			local_record;

    /*
    ** Check input parameters for NULLs
    */

    if (discrep_record == NULL)
    {
        return(ERROR);
    }

    /*
    ** Build and execute the SQL command to insert into the  **
    ** DISCREP_REP table using the DODBCMD macro             **
    */
	sqlprintf(sql_string,
		"select \
NORMAL_ID,\
ASSIGNED_ID,\
TITLE,\
STATUS,\
PRIORITY,\
ORIGIN_DATE,\
ORIGINATOR,\
ASSIGN_DATE,\
ASSIGNED_TO,\
DUE_DATE,\
CLOSE_DATE,\
CPC,\
THREAD,\
TIME_TO_COMPLETE,\
TROUBLE_AREA,\
TRANSITION_DATE,\
CO_ASSIGNEE\
 from %s_sdrs where NORMAL_ID like \"%s\" or ASSIGNED_ID like \"%s\"",
		Table_name,
        discrep_record->id[0], discrep_record->id[0]);
    command_filename[0] = script_filename[0] = results_filename[0] = '\0';
    if (!tmpnam(command_filename) ||
        !tmpnam(script_filename) ||
        !tmpnam(results_filename) ||
        !(tempfile = fopen(command_filename,"w")) ||
        !fprintf(tempfile,"%s\n",sql_string) ||
        fclose(tempfile) ||
        (tempfile = (FILE *)0) ||
        !sprintf(system_command,
			"%s %s <%s >%s 2>&1", SdrTool.sdrsqlcmd, SDRTOOL_DB_NAME,
            command_filename, results_filename) ||
        !(tempfile = fopen(script_filename,"w")) ||
        !fprintf(tempfile,"#!/bin/sh\n%s\n",system_command) ||
        fclose(tempfile) ||
        (tempfile = (FILE *)0) ||
        !sprintf(system_command,"/bin/sh %s", script_filename))
    {
        returnval = ERROR;
    }
    else
    {
        system_results = system(system_command);
		/*
		**	Expect each item asked for 1 per line in order queried
		**	ONLY as long as we can find "1 row(s) retrieved" at end.
		**	Firstword on each line (if not blank) is field name.
		*/
        if (access(results_filename,NOTEXIST) ||
            !(tempfile = fopen(results_filename,"r")))
		{
			returnval = ERROR;
		}
		else
		{
			memset(&local_record,0,sizeof(local_record));
			localptr = (char *)&local_record;
			bindno = 0;
			memset(sql_string,0,sizeof(sql_string));
			while (fgets(sql_string,sizeof(sql_string),tempfile) &&
				!(scanptr = strstr(sql_string,"1 row(s) retrieved")))
			{
				if (emptyline(sql_string))
					continue;
				else if (scanptr || bindings[bindno].type == NOOP_TYPE)
					break;
				linelen = strlen(sql_string);
				if ((pos = firstword(sql_string, bindings[bindno].name)) !=
					-1)
				{
					if ((parmptr=strtok(&sql_string[pos+
							strlen(bindings[bindno].name)]," \t\n")))
					{
						parmptr[strlen(parmptr)] = ' ';
						strip_whitespace(parmptr);
						switch (bindings[bindno].type)
						{
						case UCHAR_TYPE:
							if (!emptyline(parmptr) &&
								strncmp(parmptr,"null",4))
							{
								sscanf(parmptr,"%u",&tmpuint);
								localptr[bindings[bindno].offset] =
									(unsigned char)tmpuint;
							}
							else
								localptr[bindings[bindno].offset] =
									(unsigned char) '\0';
							break;
						case CHAR_TYPE:
							if (!emptyline(parmptr) &&
								strncmp(parmptr,"null",4))
							{
								strncpy(&localptr[bindings[bindno].offset],
									parmptr,
									bindings[bindno].size);
								localptr[bindings[bindno].offset+
									bindings[bindno].size] = '\0';
							}
							else
								localptr[bindings[bindno].offset] = '\0';
							break;
						case SHORT_TYPE:
							tempshort = (short *)
								&localptr[bindings[bindno].offset];
							if (!emptyline(parmptr) &&
								strncmp(parmptr,"null",4))
								sscanf(parmptr,"%hd",
									tempshort);
							else
								*tempshort = 0;
							break;
						case INT_TYPE:
							tempint = (int *)
								&localptr[bindings[bindno].offset];
							if (!emptyline(parmptr) &&
								strncmp(parmptr,"null",4))
								sscanf(parmptr,"%d", tempint);
							else
								*tempint = 0;
							break;
						case ULONG_TYPE:
							tempulong = (unsigned long *)
								&localptr[bindings[bindno].offset];
							if (!emptyline(parmptr) &&
								strncmp(parmptr,"null",4))
								sscanf(parmptr,"%ul", tempulong);
							else
								*tempulong = 0;
							break;
						default:
							break;
						}
					}
					++bindno;
				} /* end if */
				else /* it's not empty & they're not in order: error */
				{
					char	testmessage[256];
					char	*editcommand;
        			if (!(editcommand = (char *)getenv(SDREDITOR)))
                		editcommand = "/usr/local/bin/xvi";
					sprintf(testmessage,"%s %s",
						editcommand, results_filename);
					system(testmessage);
					break;
				}
				memset(sql_string,0,linelen);
			} /* end while */
#ifdef	__STDC__
			*discrep_record = local_record;
#else	/* __STDC__ */
			memcpy(discrep_record, &local_record, sizeof(local_record));
#endif	/* __STDC__ */
		} /* end else */
	}
    if (tempfile)
        fclose(tempfile);
#define	DELETE_THEM
#ifdef	DELETE_THEM
    if (command_filename[0] && !access(command_filename,NOTEXIST))
        remove(command_filename);
    if (script_filename[0] && !access(script_filename,NOTEXIST))
        remove(script_filename);
    if (results_filename[0] && !access(results_filename,NOTEXIST))
        remove(results_filename);
#else  /* DELETE_THEM */
	printf("%s\n\
command_filename [%s]\n\
script_filename  [%s]\n\
results_filename [%s]\n",
		__FILE__, command_filename, script_filename, results_filename);
#endif /* DELETE_THEM */
	return(returnval);
} /* end function: get_discrep_rec */  

/* end file : get_discrep_rec.c */ 
 
