/*****************************************************************************
* @(#)create.abc_sdrs.tbl.sql	1.5 5/12/95 SyddWare
*
* Table Name:      abc_sdrs
*
* Description:    
*   This code creates the abc_sdrs table which will be used to by CM to 
*   track discrepancy reports.
*
* Notes:           
*
* Revision History
*   DATE    S/W ENGINEER     DESCRIPTION OF WORK
* --------  ---------------  ------------------------------------------------
* 05/12/95  J.R.(Sydd)Souza  Created
******************************************************************************/

create table abc_sdrs    

(NORMAL_ID        char(6)  not null,      /*N#####, unique, # is 0...9 */
 ASSIGNED_ID      char(6)  not null,      /*N#####, normal(matches field above)
                                            S#####, software           
                                            H#####, hardware           
                                            D#####, documentation      
                                            T#####, tool     or        
                                            V#####, vendor             */
 TITLE            varchar(50)  not null,  /* whatever                  */
 STATUS           smallint     not null,  /* 1 - New/CCB1, 2 - Assess,
                                             3 - Assigned, 4 - CCB2, 
                                             5 - Resolved, 6 - Rejected,
                                             7 - CM Eval,  8 - Promote */
 PRIORITY         smallint     not null,  /* 1 - highest priority,
                                             2 - requirement conflict,
                                             3 - functional bug,
                                             4 - minor problem,
                                             5 - out of scope          */
 ORIGIN_DATE      smalldatetime  not null, /* mmm dd yyyy HH:MM        */
 ORIGINATOR       varchar(25)    not null,
 ASSIGN_DATE      smalldatetime      null, /* mmm dd yyyy HH:MM        */
 ASSIGNED_TO      varchar(25)        null,
 DUE_DATE         smalldatetime      null, /* mmm dd yyyy HH:MM        */
 CLOSE_DATE       smalldatetime      null, /* mmm dd yyyy HH:MM        */
 CPC              char(2)        not null, /* 2 char CPC mnemonic      */
 THREAD           varchar(30)        null, /* thread                   */
 TIME_TO_COMPLETE int                null, /* in hundreths of hours    */
 TROUBLE_AREA     varchar(75)    not null, /* UNKNOWN or user specified*/
 TRANSITION_DATE  smalldatetime      null, /* mmm dd yyyy HH:MM        */
 CO_ASSIGNEE      varchar(25)        null
)

/*****************************************************************************
*
* Index Name:      abc_sdrs_CL_U
*
* Description:     
*   This code creates a clustered index on the 
*   abc_sdrs table.  
*
* Notes:           
*
*   See the Transact_SQL User's Guide on creating clustered indexes.
*
******************************************************************************/
create unique clustered index abc_sdrs_CL_U
on abc_sdrs(NORMAL_ID)

/*****************************************************************************
*
* Index Name:      abc_sdrs_N1_U
*
* Description:     
*   This code creates an index on the 
*   abc_sdrs table.  
*
* Notes:           
*
*   See the Transact_SQL User's Guide on creating clustered indexes.
*
******************************************************************************/
create unique index abc_sdrs_N1_U
on abc_sdrs(ASSIGNED_ID)

grant all on abc_sdrs to sdrtool
