#ifndef	lint
static char sccsid[] = "@(#)update_discrep_rec.c	1.4 6/25/94 SyddWare";
#endif
/*****************************************************************************
* Classification:    UNCLASSIFIED
*
* Project:           SDRTOOL
*
* Module Name:       update_discrep_rec.c 
*
* Description:       
*   update_discrep_rec updates a row into the DISCREP_REP table.
*
* Calling Sequence:  
*   int status = update_discrep_rec      
*                        ( SDR_T  *discrep_record );
*
*   Inputs:
*       discrep_record            - pointer to a structure containing the 
*                                   record to be inserted.
*
*   Outputs: 
*     None
*
*   Returns:
*       status - returns the contents of the dbretstat member 
*                of the dbproc structure.  
*                If the record was successfully inserted dbretstat = SUCCESS.
*                If the record was not successfully inserted 
*                dbretstat != SUCCESS and the database administrator will have 
*                to examine the code to determine the cause of the failure.
*
* Error Conditions and Actions:
*   if sqlprintf, or dbcmd, or dbsqlexec fail,
*   an alarm is sent and ERROR is returned to the calling program.
*
* Side Effects:      
*
*   If successful, inserts one row in the DISCREP_REP table.
*
* Limitations: 
*   The application program must call login_to_database, to obtain
*   a DBPROCESS structure, before calling this function. 
*
* Notes: 
*   N/A
*
* Revision History
*   DATE    S/W ENGINEER     DESCRIPTION OF WORK
* --------  ---------------  ------------------------------------------------
* 10-06-93  R.K.Montgomery   Initial Creation of C Routine
******************************************************************************/

/*
** Include Files
*/

#include <stdlib.h>		/* Prototype for strcpy                   */
#include "sdr_.h"

/*
** Function update_discrep_rec
*/

int update_discrep_rec ( discrep_record )
	SDR_T *discrep_record;
{
    /* 
    **  Variable Definitions and Declarations
    **
    **************************************************************************
    **************************************************************************
    **   sql_string             - string containing stored procedure exec   **
    **                            command                                   ** 
    **************************************************************************
    */

	FILE	*tempfile = (FILE *)0;
	int		 system_results = 0;
	int		 returnval = SUCCESS;
	char	 sql_string[MAX_SQL_STRING_LEN];          
	char	 system_command[1024];
	char	 command_filename[L_tmpnam];
	char	 script_filename[L_tmpnam];
	char	 results_filename[L_tmpnam];

    /*
    ** Check input parameters for NULLs
    */

    if (discrep_record == NULL)
    {
        return(ERROR);
    }

    /*
    ** Build and execute the SQL command to insert into the  **
    ** DISCREP_REP table using the DODBCMD macro             **
    */
	sqlprintf(sql_string, "\
update %s_sdrs\n\
set    NORMAL_ID        = ~s,\n\
       ASSIGNED_ID      = ~s,\n\
       TITLE            = ~s,\n\
       STATUS           = %d,\n\
       PRIORITY         = %d,\n\
       ORIGIN_DATE      = ~s,\n\
       ORIGINATOR       = ~s,\n\
       ASSIGN_DATE      = ~s,\n\
	   ASSIGNED_TO      = ~s,\n\
       DUE_DATE         = ~s,\n\
       CLOSE_DATE       = ~s,\n\
       CPC              = ~s,\n\
       THREAD           = ~s,\n\
       TIME_TO_COMPLETE = %d,\n\
       TROUBLE_AREA     = ~s,\n\
       TRANSITION_DATE  = ~s\n\
where  NORMAL_ID = ~s\n\
",
		"CIDS",
        discrep_record->id[0],
        discrep_record->id[1],
        discrep_record->title,
        discrep_record->status,      
        discrep_record->priority,      
		discrep_record->originated,
		discrep_record->originator,
		discrep_record->assigned,
		discrep_record->assigned_to,
		discrep_record->due,
		discrep_record->closed,
        discrep_record->cpc_mnemonic,
        discrep_record->thread,
        discrep_record->hours,
        discrep_record->trouble_area,
        discrep_record->transition,
        discrep_record->id[0]);

	command_filename[0] = script_filename[0] = results_filename[0] = '\0';
	if (!tmpnam(command_filename) ||
		!tmpnam(script_filename) ||
		!tmpnam(results_filename) ||
		!(tempfile = fopen(command_filename,"w")) ||
		!fprintf(tempfile,"%s\n",sql_string) ||
		fclose(tempfile) ||
		(tempfile = (FILE *)0) ||
		!sprintf(system_command,"%s %s <%s >%s 2>&1",
			SdrTool.sdrsqlcmd,
        	SDRTOOL_DB_NAME,
			command_filename, results_filename) ||
		!(tempfile = fopen(script_filename,"w")) ||
		!fprintf(tempfile,"#!/bin/sh\n%s\n",system_command) ||
		fclose(tempfile) ||
		(tempfile = (FILE *)0) ||
		!sprintf(system_command,"/bin/sh %s", script_filename))
	{
		returnval = ERROR;
	}
	else
	{
		system_results = system(system_command);
		if (access(results_filename, NOTEXIST) ||
			!(tempfile = fopen(results_filename,"r")))
		{
			returnval = ERROR;
		}
		else
		{
			returnval = ERROR;
			while (returnval == ERROR &&
				fgets(sql_string,sizeof(sql_string),tempfile))
				if (strstr(sql_string,"1 row(s) updated"))
					returnval = SUCCESS;
		}
	}
	if (tempfile)
		fclose(tempfile);
#define	DELETE_THEM
#ifdef DELETE_THEM
	if (command_filename[0] && !access(command_filename,NOTEXIST))
		remove(command_filename);
	if (script_filename[0] && !access(script_filename,NOTEXIST))
		remove(script_filename);
	if (results_filename[0] && !access(results_filename,NOTEXIST))
		remove(results_filename);
#endif /* DELETE_THEM */
	return(returnval);
} /* end function: update_discrep_rec */  

/* end file:update_discrep_rec.c*/ 
